@@ -9,8 +9,6 @@
 #include "pobject.h"
 #include "pabstract.h"
 
-#if HAVE_GENERATORS
-
 
 /* this structure not exported by iterobject.h */
 typedef struct {
@@ -34,12 +32,10 @@ EXTERNVAR source_virtual_t psyco_computed_seqiter;
 /* !! consumes a ref on 'seq' */
 EXTERNFN vinfo_t* PsycoSeqIter_NEW(PsycoObject* po, vinfo_t* seq);
 
-inline vinfo_t* PsycoSeqIter_New(PsycoObject* po, vinfo_t* seq)
+PSY_INLINE vinfo_t* PsycoSeqIter_New(PsycoObject* po, vinfo_t* seq)
 {
 	vinfo_incref(seq);
 	return PsycoSeqIter_NEW(po, seq);
 }
 
-
-#endif /* HAVE_GENERATORS */
 #endif /* _PSY_ITEROBJECT_H */
