@@ -14,12 +14,6 @@
 #include <frameobject.h>
 
 
-/*** the PsycoCode_Xxx interface requires Python >= 2.2.2 ***/
-#define HAVE_DYN_COMPILE     HAVE_PYTHON_SUPPORT
-
-#if HAVE_DYN_COMPILE
-
-
 /* Basic interface to compile the given code object.  Return the
    code buffer or Py_None if this code cannot be compiled.
    Never sets an exception. */
@@ -41,8 +35,6 @@ EXTERNFN PyObject* PsycoCode_CompileFrame(PyFrameObject* f, int recursion);
    was compiled. */
 EXTERNFN bool PsycoCode_Run(PyObject* codebuf, PyFrameObject* f, bool entering);
 
-#endif /* HAVE_DYN_COMPILE */
-
 
 /* Find a frame. If 'o' is an integer, it is the 'o'th frame (0=top).
    If 'o' was returned by a previous call to psyco_get_frame(), find
@@ -65,7 +57,7 @@ struct stack_frame_info_s {
 	PyCodeObject* co;
 	PyObject* globals;  /* NULL if not compile-time */
 };
-inline stack_frame_info_t* finfo_last(stack_frame_info_t* finfo) {
+PSY_INLINE stack_frame_info_t* finfo_last(stack_frame_info_t* finfo) {
 #if NEED_STACK_FRAME_HACK
 	if (finfo->link_stack_depth < 0) finfo -= finfo->link_stack_depth;
 #endif
