@@ -17,28 +17,6 @@ static PyCFunction cimpl_apply;
 static PyCFunction cimpl_divmod;
 
 
-#if HAVE_METH_O
-# define METH_O_WRAPPER(name, self, arg)    do { } while (0)  /* nothing */
-#else
-# define METH_O_WRAPPER(name, self, arg)    do {                                \
-    if (PsycoTuple_Load(arg) != 1)                                              \
-      return psyco_generic_call(po, cimpl_ ## name, CfReturnRef|CfPyErrIfNull,  \
-                                "vv", self, arg);                               \
-    arg = PsycoTuple_GET_ITEM(arg, 0);                                          \
-} while (0)
-#endif
-
-#if HAVE_METH_NOARGS
-# define METH_NOARGS_WRAPPER(name, self, arg)  do { } while (0)  /* nothing */
-#else
-# define METH_NOARGS_WRAPPER(name, self, arg)  do {                             \
-    if (PsycoTuple_Load(arg) != 0)                                              \
-      return psyco_generic_call(po, cimpl_ ## name, CfReturnRef|CfPyErrIfNull,  \
-                                "vv", self, arg);                               \
-} while (0)
-#endif
-
-
 static vinfo_t* get_len_of_range(PsycoObject* po, vinfo_t* lo, vinfo_t* hi
 				 /*, vinfo_t* step == 1 currently*/)
 {
@@ -130,26 +108,22 @@ static vinfo_t* pbuiltin_xrange(PsycoObject* po, vinfo_t* vself, vinfo_t* vargs)
 				  "lv", NULL, vargs);
 }
 
-#if NEW_STYLE_TYPES
-static vinfo_t* prange_new(PsycoObject* po, vinfo_t* vtype,
+static vinfo_t* prange_new(PsycoObject* po, PyTypeObject* type,
 			   vinfo_t* vargs, vinfo_t* vkw)
 {
 	/* for Python >= 2.3, where __builtin__.xrange is a type */
 	vinfo_t* istart;
 	vinfo_t* ilen;
+	psyco_assert(type == &PyRange_Type);   /* no subclassing xrange */
 	if (parse_range_args(po, vargs, &istart, &ilen)) {
 		return PsycoXRange_NEW(po, istart, ilen);
 	}
 	if (PycException_Occurred(po))
 		return NULL;
-	return psyco_generic_call(po, PyRange_Type.tp_new,
+	return psyco_generic_call(po, type->tp_new,
 				  CfReturnRef|CfPyErrIfNull,
-				  "lvv", &PyRange_Type,
-				  vargs, vkw);
+				  "lvv", type, vargs, vkw);
 }
-#else
-# define prange_new  NULL
-#endif
 
 static vinfo_t* pbuiltin_chr(PsycoObject* po, vinfo_t* vself, vinfo_t* vargs)
 {
@@ -185,8 +159,6 @@ static vinfo_t* pbuiltin_chr(PsycoObject* po, vinfo_t* vself, vinfo_t* vargs)
 static vinfo_t* pbuiltin_ord(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 {
 	vinfo_t* result;
-	METH_O_WRAPPER(ord, vself, vobj);
-
 	if (!PsycoCharacter_Ord(po, vobj, &result))
 		return NULL;
 	
@@ -199,15 +171,19 @@ static vinfo_t* pbuiltin_ord(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 
 static vinfo_t* pbuiltin_id(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 {
-	METH_O_WRAPPER(id, vself, vobj);
+#if HAVE_NEGATIVE_IDS   /* Python < 2.5 */
 	return PsycoInt_FromLong(vobj);
+#else
+	/* XXX fall-back for now */
+	return psyco_generic_call(po, PyLong_FromVoidPtr,
+				  CfReturnRef|CfPyErrIfNull,
+				  "v", vobj);
+#endif
 }
 
 static vinfo_t* pbuiltin_len(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 {
 	vinfo_t* result;
-	METH_O_WRAPPER(len, vself, vobj);
-	
 	result = PsycoObject_Size(po, vobj);
 	if (result != NULL)
 		result = PsycoInt_FROM_LONG(result);
@@ -216,7 +192,6 @@ static vinfo_t* pbuiltin_len(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 
 static vinfo_t* pbuiltin_abs(PsycoObject* po, vinfo_t* vself, vinfo_t* vobj)
 {
-	METH_O_WRAPPER(abs, vself, vobj);
 	return PsycoNumber_Absolute(po, vobj);
 }
 
@@ -293,10 +268,10 @@ void psyco_bltinmodule_init(void)
 
 	DEFMETA( range,		METH_VARARGS );
 	DEFMETA( chr,		METH_VARARGS );
-	DEFMETA( ord,		HAVE_METH_O ? METH_O : METH_VARARGS );
-	DEFMETA( id,		HAVE_METH_O ? METH_O : METH_VARARGS );
-	DEFMETA( len,		HAVE_METH_O ? METH_O : METH_VARARGS );
-	DEFMETA( abs,		HAVE_METH_O ? METH_O : METH_VARARGS );
+	DEFMETA( ord,		METH_O);
+	DEFMETA( id,		METH_O);
+	DEFMETA( len,		METH_O);
+	DEFMETA( abs,		METH_O);
 	DEFMETA( apply,		METH_VARARGS );
 	DEFMETA( divmod,	METH_VARARGS );
 	cimpl_xrange = Psyco_DefineModuleC(md, "xrange", METH_VARARGS,
